#<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
#                                                                        <
# Package:                 EMMIX-FDR                                      <
# Version: 0.0-3
# Depends: R (>=2.0)                                                      <  
# Authors: G.J. McLachlan, R.W. Bean, L. Ben-Tovim Jones                  <
#
# Maintainer: Kui Wang <kwang@maths.uq.edu.au>                           <
#                                                                        <
#                                                                        <
# Last modified on 31 May, 2007                                         <
#
#
#
#                                                                        <
# Based on Richard Bean's R scripts of FDR. 
# emmixfdr.dll is based on EMMIX.f.                                       <
# http://maths.uq.edu.au/~gjm                                            <
#----------------------------------------------------------------------                                                                        <

#-------------------------
# beginning of code
#-------------------------
#-----------------------------------------------------------------------

myrowVars<-function(x=as.matrix(x),n=nrow(x))
{

vecvar<-rep(0,n)
for(i in 1:n)
	vecvar[i]<-var(x[i,],na.rm=T)
vecvar

}

#normalize the rows
DoRows<-function(dat)
{
dat<-as.matrix(dat)
(dat-rowMeans(dat,na.rm=T))/sqrt(myrowVars(dat))
}


#normalize the columns
DoCols<-function(dat)
{
t(DoRows(t(dat)))
}


# (Based on R Bean's R script)
tvalue.test<-function(a,len1)
{

c <- combn(ncol(a),len1)
nperm <- ncol(c)
test <- matrix(rep(0,nperm*nrow(a)),nrow=nrow(a),ncol=nperm)


for (j in 1:nperm)
{

r1<-c[,j]
r2 <- setdiff(1:ncol(a),r1)

sd1<-sqrt(apply(a[,r1],1,var))
sd2<-sqrt(apply(a[,r2],1,var))

sd <- sqrt( ( (len1-1)*sd1^2 + (len2-1)*sd2^2) / (len1+len2-2) )

mean1 <- apply(a[,r1],1,mean)
mean2 <- apply(a[,r2],1,mean)

test[,j]<-(mean1-mean2)/(sd*(1/len1 + 1/len2)^(1/2))
}
test
}

# (Based on Richard Bean's )
permpvalall <- function(a,len1,len2)
{

#
tvals <- gettpooled(a,len1,len2)
pvals <- rep(0,nrow(a))
#

test<-tvalue.test(a,len1)
nperm<-ncol(test)

for (i in 1:nrow(a))
pvals[i] <- length(which(abs(test[i,])>abs(tvals[i])))/nperm
return(pvals)
}

#(Based on Richard Bean's R script)
permpvalallpooled <- function(a,len1,len2)
{

tvals <- gettpooled(a,len1,len2)
pvals <- rep(0,nrow(a))

test<-tvalue.test(a,len1)
nperm<-ncol(test)

for (i in 1:nrow(a))
pvals[i] <- length(which(abs(c(test))>abs(tvals[i])))/nperm/nrow(a)
return(pvals)
}


# calcaulate the t-value 
# (Richard Bean)
gettpooled <- function(a,len1,len2)
{
	t <- rep(0,nrow(a))
	for (i in 1:nrow(a))
		t[i]<-t.test(a[i,1:len1],a[i,len1+1:len2],var.equal=T)$s
	return (t)
}

# calculate the p-value 
# (Richard Bean)
get_p_values.pool <- function(a,len1,len2)
{
t <- gettpooled(a,len1,len2)
p <- 1-pf(t^2,1,len1+len2-2)

return (p)
}
#
get_p_values.perm <- function(a,len1,len2)
{
permpvalallpooled(a,len1,len2)
}
#


EmmixFdr<-function(zz,mu,mvar,pai,model)
{
if(!is.loaded('emmixfdr.dll'))
dyn.load('emmixfdr.dll')

dat<-as.matrix(zz)
if( (m=ncol(dat))>1) 
stop("zz values must be a vector")

obj<-.Fortran('emmix',PACKAGE='emmixfdr',
as.double(dat),
as.integer((n=nrow(dat))),
as.integer(m),
as.integer((g=2)),
mu0   = as.double(mu),
var0  = as.double(mvar),
pai0  = as.double(pai),
mode  = as.integer(ifelse(model=='theo',1,0)),
aic   = double(1),
bic   = double(1),
loglik= double(1),
mus   = double(g*m),
vars  = double(g*m*m),
pai   = double(g),
clust = integer(n),
tao   = double(g*n))  #[-(1:10)]

dyn.unload('emmixfdr.dll')
obj

}



# hunt.theo.null(), hunt.empi.null() are based on Richard Bean's R scripts

hunt.theo.null<-function(z,debug,pi0seq=NULL,xiseq=NULL)
{

## estimate \pi_0 by looking at the number of p-values between 
## a certain number \xi and 1 -- an application of equation (21) in
## the paper

found<-list()
found$loglik<- -1e+10
i<-1
counter<-0

logliklist<-matrix(0,nrow=1000,ncol=2)
#


if(is.null(pi0seq))
{
if(is.null(xiseq))
xiseq<-seq(from=-3,to=3,by=0.1)
colnames(logliklist)<-c("xi","log-likelihood")

for(xi in xiseq)
{
pi0 <- length(which(z < xi))/(length(z)*pnorm(xi))
if(debug)
cat('\ndoing... xi = ',xi,        ',\tpi0     = ', pi0,'\n')

if (pi0 < 1& pi0>0)
{
#
mu <- mean(z)
var <- var(z)
## since m0 and v0 are 0, 1; then we use equations (19) and (20) 
## in the paper to calculate m1 and v1

mu1<- mu/(1-pi0)
v1<-(var-pi0-pi0*(1-pi0)*mu1^2)/(1-pi0)
if (v1 < 0) v1=1

objn<-EmmixFdr(z,c(0,mu1),c(1,v1),c(pi0,1-pi0),'theo')
if(length(objn)!=16) next
logliklist[i,]<-c(xi,objn$loglik)
counter<-counter+1


if(debug)
cat(
'loglik = ',round(logliklist[i,2],5),',\tpi_null = ', round(objn$pai[1],5), '\n',
'mu_alt = ',round(objn$mus[2],5),    ',\tvar_alt = ', round(objn$vars[2],5),'\n',
'...finished\n')


if(objn$loglik>found$loglik)
found<-objn; 
}
i<-i+1
} #endfor

}
else
{
#	if(is.null(pi0seq))
#	pi0seq<-seq(from=0.1,to=.98,by=0.05)
colnames(logliklist)<-c("pi0","log-likelihood")

for(pi0 in pi0seq)
{
if (pi0 < 1& pi0>0)
{

if(debug)
cat('\ndoing... pi0 = ', pi0,'\n')

#
mu <- mean(z)
var <- var(z)
## since m0 and v0 are 0, 1; then we use equations (19) and (20) 
## in the paper to calculate m1 and v1

mu1<- mu/(1-pi0)
v1<-(var-pi0-pi0*(1-pi0)*mu1^2)/(1-pi0)
if (v1 < 0) v1=1

objn<-EmmixFdr(z,c(0,mu1),c(1,v1),c(pi0,1-pi0),'theo')
if(length(objn)!=16) next
logliklist[i,]<-c(pi0,objn$loglik)
counter<-counter+1


if(debug)
cat(
'loglik = ',round(logliklist[i,2],5),',\tpi_null = ', round(objn$pai[1],5), '\n',
'mu_alt = ',round(objn$mus[2],5),    ',\tvar_alt = ', round(objn$vars[2],5),'\n',
'...finished\n')


if(objn$loglik>found$loglik)
found<-objn; 
}
i<-i+1
} #endfor

} #endif

if(counter==0) 
stop("for given initials, none of them converges, try other initials!")
found$logliklist<-logliklist[logliklist[,2]<0,]
found
}

#

hunt.empi.null<-function(z,debug,pi0seq=NULL)
{
if(length(z)>20000) stop("in this version, number of genes must be smaller than 20000")

found<-list()
found$loglik<- -Inf
logliklist<-matrix(0,nrow=1000,ncol=2)
colnames(logliklist)<-c("pi0","log-likelihood")


i<-1
counter<-0
#
if(is.null(pi0seq))
pi0seq<-seq(from=0.1,to=.98,by=0.05)

for(pi0 in pi0seq)
{
if (pi0 < 1& pi0>0)
{
if(debug)
cat('\ndoing... pi0 = ', pi0,'\n')

a1 <- pi0*length(z)

if (a1 < 2) next;
if  (length(z) == ceiling(a1+1)) next;

zs <- sort(z)
mu0<-(mean(zs[1:a1]))
v0<-(var(zs[1:a1]))
mu1<-(mean(zs[(a1+1):length(z)]))
v1<-(var(zs[(a1+1):length(z)]))

if(debug)
cat('\ndoing... pi0 = ',pi0)
objn<-EmmixFdr(z,c(mu0,mu1),c(v0,v1),c(pi0,1-pi0),'empi')
if(length(objn)!=16) next
#
logliklist[i,]<-c(pi0,objn$loglik)
counter<-counter+1
if(debug)
cat('\n',
'loglik = ',round(logliklist[i,2],5),',\tpi_null =',round(objn$pai[1],5),'\n',
'mu_null= ',round(objn$mus[1],5),    ',\tvar_null=',round(objn$vars[1],5),'\n',
'mu_alt = ',round(objn$mus[2],5),    ',\tvar_alt =',round(objn$vars[2],5),'\n',
'...finished\n')

if(objn$loglik>found$loglik)
found<-objn; 
}
i<-i+1

} #endfor
if(counter==0) 
stop("for given initials, none of them converges, try other initials!")

found$logliklist<-logliklist[logliklist[,2]<0,]
found
}
#
#-----------------------------------------------------
# FdrTable() is based on Richard Bean's script
#
FdrTable<-function(tx,xseq)
{
colnames <- c("a","b","c","d","fdr","fpr","fnr","fndr","sens","spec")
xm <- matrix(nrow=length(xseq),ncol=10,
dimnames=list(paste('c0=',xseq,sep=''), colnames))
xmrow <- 1
for (c0 in xseq)
{
#
a <- sum (tx[tx > c0])

b <- sum (tx[tx <= c0])

c <- sum ( 1-tx[tx > c0])
d <- sum ( 1-tx[tx <= c0])
fpr <- b/(a+b)
fnr <- c/(c+d)

fdr <- b/(b+d)

fndr <- c/(a+c)
sens <- d/(c+d)
spec <- a/(a+b)

#
xm[xmrow,] <- c(a,b,c,d,fdr,fpr,fnr,fndr,sens,spec)
xmrow <- xmrow + 1
}
xm
}
#-----------------------------------------------------
#
GetEmmixFdr<- function(
file=''            #  file name             
,m1                #  number of tisssues in class I
,m2                #  number of tisssues in class I
,model='theo'      # 'theo'(default), or 'empi'
,method='pool'     # 'perm', or 'pool'(default)
,debug=0           #  1, print noise, 0 quite
,xiseq=NULL        #  xi sequence
,pi0seq=NULL       #  pi0 sequenxes
)
{
#
dat<-read.table(file)
# 

ret<-get.fdr(dat,m1,m2,model,method,debug,xiseq ,pi0seq)



cat('\n-------------------------------------\n')
print(ret$xm)
cat('\n-------------------------------------\n')

#-----------------------------------------------------

cat('\n'  ,'the null  component is N(', ret$emobj$mus[1], ',',ret$emobj$vars[1],')')
cat('\n\n','the alternative component is N(',ret$emobj$mus[2],',',ret$emobj$vars[2],')')
cat('\n\n','the mixing propotion pi_null is:', ret$emobj$pai[1])
cat('\n\n','the best initial pi0 tried is: ', ret$emobj$pai0[1])

cat('\n\n-------------------------------------\n')
x11()
plot(ret$emobj$logliklist)

ret
}

GetFdr<- function(
dat                #  file name             
,m1                #  number of tisssues in class I
,m2                #  number of tisssues in class I
,model='theo'      # 'theo'(default), or 'empi'
,method='pool'     # 'perm', or 'pool'(default)
,debug=0           #  1, print noise, 0 quite
,xiseq=NULL        #  xi sequence
,pi0seq=NULL       #  pi0 sequenxes
)
{
# 

ret<-get.fdr(dat,m1,m2,model,method,debug,xiseq ,pi0seq)



cat('\n-------------------------------------\n')
print(ret$xm)
cat('\n-------------------------------------\n')

#-----------------------------------------------------

cat('\n'  ,'the null  component is N(', ret$emobj$mus[1], ',',ret$emobj$vars[1],')')
cat('\n\n','the alternative component is N(',ret$emobj$mus[2],',',ret$emobj$vars[2],')')
cat('\n\n','the mixing propotion pi_null is:', ret$emobj$pai[1])
cat('\n\n','the best initial pi0 tried is: ', ret$emobj$pai0[1])

cat('\n\n-------------------------------------\n')
x11()
plot(ret$emobj$logliklist)

ret
}


get.fdr<- function(
dat            #  file name             
,m1                #  number of tisssues in class I
,m2                #  number of tisssues in class I
,model='theo'      # 'theo'(default), or 'empi'
,method='pool'     # 'perm', or 'pool'(default)
,debug=0           #  1, print noise, 0 quite
,xiseq=NULL        #  xi sequence
,pi0seq=NULL       #  pi0 sequenxes
)
{
#
dat<-as.matrix(dat)
#
#

pp<-switch(method,
		pool=get_p_values.pool(dat,m1,m2),
		perm=get_p_values.perm(dat,m1,m2),
		)

# to avoid inf, correct pp values
#pp[pp==0]<-1e-16
#pp[pp==1]<-1-(1e-16)
#zz <- qnorm(1-pp)

#zz <-  ifelse(pp <= (1e-16),-qnorm(pp), qnorm(1-pp))
R.min <- (1e-323)

zz <-(  ifelse( (pp <= (1e-16)) & (pp >= R.min) ,-qnorm(pp), 0)
				+ ifelse(  pp <  R.min,runif(length(pp),min=20,max=39),0)
				+ ifelse(  pp <  1e-16 ,0,qnorm(1-pp)))


#
#

emobj<-switch(model,
	 theo=hunt.theo.null(zz,debug,pi0seq,xiseq),
       empi=hunt.empi.null(zz,debug,pi0seq)	)

#
tx <- matrix(emobj$tao,ncol=2)[,1]
xseq <- seq(0.1,0.5,0.1)
xm<-FdrTable(tx,xseq)


zdat<-cbind(1:length(zz),pp,zz,tx)
colnames(zdat)<-c("id","p-value","z-value","posterior probability")

list(emobj=emobj,xm=xm,zdat=zdat,tx=tx)
}

EmmixFdrAnalyze<-function(fdrobj,cseq=NULL,outfile=NULL)
{
if(is.null(outfile))
outfile<-"EMMIX-FDR.txt"
#
write.table(fdrobj$zdat,outfile)
cat("\n\nImportant Note: \n\nBeware of that gene IDs (ordinals),p-values, z-values,", 
'\n\nand posterior probabilities for each gene are written to\n\n\'',outfile, 
'\', which is to be used for further analysis \n\naccording to p-values or z-values or posterior probabilities.',
'\n\nFor example, read in \'',outfile,'\' at Excel and sort it to filter out \n\nthose differentially expressed genes just recognized by EMMIX-FDR\n\n\n')

# print more c0s, fdrs
if(!is.null(cseq))
{
cat("\n------------------------------------\n\n")
print(FdrTable(fdrobj$tx,cseq))
cat("\n------------------------------------\n\n")
}
#
} #end


GetFdrFromPvalue<- function(
pp                 #  p-value             
,model='theo'      # 'theo'(default), or 'empi'
,method='pool'     # 'perm', or 'pool'(default)
,debug=0           #  1, print noise, 0 quite
,xiseq=NULL        #  xi sequence
,pi0seq=NULL       #  pi0 sequenxes
)
{
#
# to avoid inf, correct pp values
pp[pp==0]<-1e-16
pp[pp==1]<-1-(1e-16)
#
zz <- qnorm(1-pp)
#
#

emobj<-switch(model,
	 theo=hunt.theo.null(zz,debug,pi0seq,xiseq),
       empi=hunt.empi.null(zz,debug,pi0seq),
	)

#
tx <- matrix(emobj$tao,ncol=2)[,1]
xseq <- seq(0.1,0.5,0.1)
xm<-FdrTable(tx,xseq)

cat('\n-------------------------------------\n')
print(xm)
cat('\n-------------------------------------\n')

#-----------------------------------------------------

cat('\n'  ,'the null  component is N(',emobj$mus[1],',',emobj$vars[1],')')
cat('\n\n','the alternative component is N(',emobj$mus[2],',',emobj$vars[2],')')
cat('\n\n','the mixing propotion pi_null is:', emobj$pai[1])
cat('\n\n','the best initial pi0 tried is: ', emobj$pai0[1])

cat('\n\n-------------------------------------\n')

zdat<-cbind(1:length(zz),pp,zz,tx)
colnames(zdat)<-c("id","p-value","z-value","posterior probability")
x11()
plot(emobj$logliklist)

list(emmix=emobj,fdr=xm,zdat=zdat,tx=tx)
}

GetFdrFromZvalue<- function(
zz                 #  p-value             
,model='theo'      # 'theo'(default), or 'empi'
,method='pool'     # 'perm', or 'pool'(default)
,debug=0           #  1, print noise, 0 quite
,xiseq=NULL        #  xi sequence
,pi0seq=NULL       #  pi0 sequenxes
)
{
#
# to avoid inf, correct pp values
pp<-1-pnorm(abs(zz))
#

emobj<-switch(model,
	 theo=hunt.theo.null(zz,debug,pi0seq,xiseq),
       empi=hunt.empi.null(zz,debug,pi0seq),
	)

#
tx <- matrix(emobj$tao,ncol=2)[,1]
xseq <- seq(0.1,0.5,0.1)
xm<-FdrTable(tx,xseq)

cat('\n-------------------------------------\n')
print(xm)
cat('\n-------------------------------------\n')

#-----------------------------------------------------

cat('\n'  ,'the null  component is N(',emobj$mus[1],',',emobj$vars[1],')')
cat('\n\n','the alternative component is N(',emobj$mus[2],',',emobj$vars[2],')')
cat('\n\n','the mixing propotion pi_null is:', emobj$pai[1])
cat('\n\n','the best initial pi0 tried is: ', emobj$pai0[1])

cat('\n\n-------------------------------------\n')

x11()
plot(emobj$logliklist)

emobj

}


#-------------------------
# end of code
#-------------------------
